/**
 * Copyright (c) 2013 Andre Ricardo Schaffer
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package com.github.wiselenium.core;

import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.internal.WrapsDriver;
import org.openqa.selenium.internal.WrapsElement;

/**
 * Utility class to unwrap WebElements out of Elements and WebDrivers out of Pages.
 * 
 * @author Andre Ricardo Schaffer
 * @since 0.1.0
 */
public final class WiseUnwrapper {
	
	private WiseUnwrapper() {}
	
	/**
	 * Unwraps the WebDriver of a page. <br/>
	 * In order to do so, the page must implement WrapsDriver. (Notice: if the page was created
	 * using the WisePageFactory with a no-arg constructor, it is in fact a proxy that implements it
	 * already.)
	 * 
	 * @param page The page.
	 * @return The WebDriver decorated by the page.
	 * @since 0.1.0
	 */
	public static WebDriver unwrapWebDriver(Object page) {
		if (page instanceof WrapsDriver) return ((WrapsDriver) page).getWrappedDriver();
		throw new WebDriverNotWrappedException(page);
	}
	
	/**
	 * Unwraps the WebElement of an Element. <br/>
	 * In order to do so, the element must implement WrapsElement. (Notice: wiselenium elements'
	 * proxies already implement it.)
	 * 
	 * @param element The element.
	 * @return The WebElement decorated by the Element.
	 * @since 0.1.0
	 */
	public static WebElement unwrapWebElement(Object element) {
		if (element instanceof WrapsElement) return ((WrapsElement) element).getWrappedElement();
		throw new WebElementNotWrappedException(element);
	}
	
}
